@@ -16,6 +16,7 @@ package prometheus
 
 import (
 	"bytes"
+	"compress/gzip"
 	"fmt"
 	"reflect"
 	"strings"
@@ -31,7 +32,7 @@ import (
 	"github.com/mitchellh/hashstructure"
 	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
-	appsv1 "k8s.io/api/apps/v1beta2"
+	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 	extensionsobj "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	apiextensionsclient "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset"
@@ -64,6 +65,7 @@ type Operator struct {
 
 	promInf cache.SharedIndexInformer
 	smonInf cache.SharedIndexInformer
+	pmonInf cache.SharedIndexInformer
 	ruleInf cache.SharedIndexInformer
 	cmapInf cache.SharedIndexInformer
 	secrInf cache.SharedIndexInformer
@@ -131,24 +133,29 @@ func (labels *Labels) Set(value string) error {
 
 // Config defines configuration parameters for the Operator.
 type Config struct {
-	Host                         string
-	KubeletObject                string
-	TLSInsecure                  bool
-	TLSConfig                    rest.TLSClientConfig
-	ConfigReloaderImage          string
-	PrometheusConfigReloader     string
-	AlertmanagerDefaultBaseImage string
-	PrometheusDefaultBaseImage   string
-	ThanosDefaultBaseImage       string
-	Namespaces                   []string
-	Labels                       Labels
-	CrdGroup                     string
-	CrdKinds                     monitoringv1.CrdKinds
-	EnableValidation             bool
-	LocalHost                    string
-	LogLevel                     string
-	LogFormat                    string
-	ManageCRDs                   bool
+	Host                          string
+	KubeletObject                 string
+	TLSInsecure                   bool
+	TLSConfig                     rest.TLSClientConfig
+	ConfigReloaderImage           string
+	ConfigReloaderCPU             string
+	ConfigReloaderMemory          string
+	PrometheusConfigReloaderImage string
+	AlertmanagerDefaultBaseImage  string
+	PrometheusDefaultBaseImage    string
+	ThanosDefaultBaseImage        string
+	Namespaces                    []string
+	DeniedNamespaces              []string
+	Labels                        Labels
+	CrdGroup                      string
+	CrdKinds                      monitoringv1.CrdKinds
+	EnableValidation              bool
+	LocalHost                     string
+	LogLevel                      string
+	LogFormat                     string
+	ManageCRDs                    bool
+	PromSelector                  string
+	AlertManagerSelector          string
 }
 
 type BasicAuthCredentials struct {
@@ -178,6 +185,14 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 		return nil, errors.Wrap(err, "instantiating monitoring client failed")
 	}
 
+	if _, err := labels.Parse(conf.PromSelector); err != nil {
+		return nil, errors.Wrap(err, "can not parse prometheus selector value")
+	}
+
+	if _, err := labels.Parse(conf.AlertManagerSelector); err != nil {
+		return nil, errors.Wrap(err, "can not parse alertmanager selector value")
+	}
+
 	kubeletObjectName := ""
 	kubeletObjectNamespace := ""
 	kubeletSyncEnabled := false
@@ -207,19 +222,23 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 	}
 
 	c.promInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
+		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
 			return &cache.ListWatch{
 				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+					options.LabelSelector = c.config.PromSelector
 					return mclient.MonitoringV1().Prometheuses(namespace).List(options)
 				},
-				WatchFunc: mclient.MonitoringV1().Prometheuses(namespace).Watch,
+				WatchFunc: func(options metav1.ListOptions) (watch.Interface, error) {
+					options.LabelSelector = c.config.PromSelector
+					return mclient.MonitoringV1().Prometheuses(namespace).Watch(options)
+				},
 			}
 		}),
 		&monitoringv1.Prometheus{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
 	c.smonInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
+		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
 			return &cache.ListWatch{
 				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 					return mclient.MonitoringV1().ServiceMonitors(namespace).List(options)
@@ -230,8 +249,20 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 		&monitoringv1.ServiceMonitor{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
+	c.pmonInf = cache.NewSharedIndexInformer(
+		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+			return &cache.ListWatch{
+				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+					return mclient.MonitoringV1().PodMonitors(namespace).List(options)
+				},
+				WatchFunc: mclient.MonitoringV1().PodMonitors(namespace).Watch,
+			}
+		}),
+		&monitoringv1.PodMonitor{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
+	)
+
 	c.ruleInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
+		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
 			return &cache.ListWatch{
 				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 					return mclient.MonitoringV1().PrometheusRules(namespace).List(options)
@@ -243,7 +274,7 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 	)
 
 	c.cmapInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
+		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
 			return &cache.ListWatch{
 				ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 					options.LabelSelector = labelPrometheusName
@@ -259,15 +290,15 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 	)
 
 	c.secrInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
-			return cache.NewListWatchFromClient(c.kclient.Core().RESTClient(), "secrets", namespace, fields.Everything())
+		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+			return cache.NewListWatchFromClient(c.kclient.CoreV1().RESTClient(), "secrets", namespace, fields.Everything())
 		}),
 		&v1.Secret{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
 
 	c.ssetInf = cache.NewSharedIndexInformer(
-		listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
-			return cache.NewListWatchFromClient(c.kclient.AppsV1beta2().RESTClient(), "statefulsets", namespace, fields.Everything())
+		listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+			return cache.NewListWatchFromClient(c.kclient.AppsV1().RESTClient(), "statefulsets", namespace, fields.Everything())
 		}),
 		&appsv1.StatefulSet{}, resyncPeriod, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc},
 	)
@@ -284,7 +315,7 @@ func New(conf Config, logger log.Logger) (*Operator, error) {
 		nsResyncPeriod = resyncPeriod
 	}
 	c.nsInf = cache.NewSharedIndexInformer(
-		listwatch.NewUnprivilegedNamespaceListWatchFromClient(c.kclient.Core().RESTClient(), c.config.Namespaces, fields.Everything()),
+		listwatch.NewUnprivilegedNamespaceListWatchFromClient(c.logger, c.kclient.CoreV1().RESTClient(), c.config.Namespaces, c.config.DeniedNamespaces, fields.Everything()),
 		&v1.Namespace{}, nsResyncPeriod, cache.Indexers{},
 	)
 
@@ -319,6 +350,7 @@ func (c *Operator) waitForCacheSync(stopc <-chan struct{}) error {
 	}{
 		{"Prometheus", c.promInf},
 		{"ServiceMonitor", c.smonInf},
+		{"PodMonitor", c.pmonInf},
 		{"PrometheusRule", c.ruleInf},
 		{"ConfigMap", c.cmapInf},
 		{"Secret", c.secrInf},
@@ -352,6 +384,11 @@ func (c *Operator) addHandlers() {
 		DeleteFunc: c.handleSmonDelete,
 		UpdateFunc: c.handleSmonUpdate,
 	})
+	c.pmonInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
+		AddFunc:    c.handlePmonAdd,
+		DeleteFunc: c.handlePmonDelete,
+		UpdateFunc: c.handlePmonUpdate,
+	})
 	c.ruleInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
 		AddFunc:    c.handleRuleAdd,
 		DeleteFunc: c.handleRuleDelete,
@@ -410,6 +447,7 @@ func (c *Operator) Run(stopc <-chan struct{}) error {
 
 	go c.promInf.Run(stopc)
 	go c.smonInf.Run(stopc)
+	go c.pmonInf.Run(stopc)
 	go c.ruleInf.Run(stopc)
 	go c.cmapInf.Run(stopc)
 	go c.secrInf.Run(stopc)
@@ -475,6 +513,7 @@ func (c *Operator) handlePrometheusUpdate(old, cur interface{}) {
 }
 
 func (c *Operator) reconcileNodeEndpoints(stopc <-chan struct{}) {
+	c.syncNodeEndpointsWithLogError()
 	ticker := time.NewTicker(3 * time.Minute)
 	defer ticker.Stop()
 	for {
@@ -482,10 +521,7 @@ func (c *Operator) reconcileNodeEndpoints(stopc <-chan struct{}) {
 		case <-stopc:
 			return
 		case <-ticker.C:
-			err := c.syncNodeEndpoints()
-			if err != nil {
-				level.Error(c.logger).Log("msg", "syncing nodes into Endpoints object failed", "err", err)
-			}
+			c.syncNodeEndpointsWithLogError()
 		}
 	}
 }
@@ -534,6 +570,13 @@ func getNodeAddresses(nodes *v1.NodeList) ([]v1.EndpointAddress, []error) {
 	return addresses, errs
 }
 
+func (c *Operator) syncNodeEndpointsWithLogError() {
+	err := c.syncNodeEndpoints()
+	if err != nil {
+		level.Error(c.logger).Log("msg", "syncing nodes into Endpoints object failed", "err", err)
+	}
+}
+
 func (c *Operator) syncNodeEndpoints() error {
 	eps := &v1.Endpoints{
 		ObjectMeta: metav1.ObjectMeta{
@@ -645,6 +688,42 @@ func (c *Operator) handleSmonDelete(obj interface{}) {
 	}
 }
 
+// TODO: Don't enque just for the namespace
+func (c *Operator) handlePmonAdd(obj interface{}) {
+	o, ok := c.getObject(obj)
+	if ok {
+		level.Debug(c.logger).Log("msg", "PodMonitor added")
+		c.triggerByCounter.WithLabelValues(monitoringv1.PodMonitorsKind, "add").Inc()
+		c.enqueueForNamespace(o.GetNamespace())
+	}
+}
+
+// TODO: Don't enque just for the namespace
+func (c *Operator) handlePmonUpdate(old, cur interface{}) {
+	if old.(*monitoringv1.PodMonitor).ResourceVersion == cur.(*monitoringv1.PodMonitor).ResourceVersion {
+		return
+	}
+
+	o, ok := c.getObject(cur)
+	if ok {
+		level.Debug(c.logger).Log("msg", "PodMonitor updated")
+		c.triggerByCounter.WithLabelValues(monitoringv1.PodMonitorsKind, "update").Inc()
+
+		c.enqueueForNamespace(o.GetNamespace())
+	}
+}
+
+// TODO: Don't enque just for the namespace
+func (c *Operator) handlePmonDelete(obj interface{}) {
+	o, ok := c.getObject(obj)
+	if ok {
+		level.Debug(c.logger).Log("msg", "PodMonitor delete")
+		c.triggerByCounter.WithLabelValues(monitoringv1.PodMonitorsKind, "delete").Inc()
+
+		c.enqueueForNamespace(o.GetNamespace())
+	}
+}
+
 // TODO: Don't enque just for the namespace
 func (c *Operator) handleRuleAdd(obj interface{}) {
 	o, ok := c.getObject(obj)
@@ -998,7 +1077,7 @@ func (c *Operator) sync(key string) error {
 	sClient := c.kclient.CoreV1().Secrets(p.Namespace)
 	_, err = sClient.Get(s.Name, metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
-		if _, err := c.kclient.Core().Secrets(p.Namespace).Create(s); err != nil && !apierrors.IsAlreadyExists(err) {
+		if _, err := c.kclient.CoreV1().Secrets(p.Namespace).Create(s); err != nil && !apierrors.IsAlreadyExists(err) {
 			return errors.Wrap(err, "creating empty config file failed")
 		}
 	}
@@ -1007,12 +1086,12 @@ func (c *Operator) sync(key string) error {
 	}
 
 	// Create governing service if it doesn't exist.
-	svcClient := c.kclient.Core().Services(p.Namespace)
+	svcClient := c.kclient.CoreV1().Services(p.Namespace)
 	if err := k8sutil.CreateOrUpdateService(svcClient, makeStatefulSetService(p, c.config)); err != nil {
 		return errors.Wrap(err, "synchronizing governing service failed")
 	}
 
-	ssetClient := c.kclient.AppsV1beta2().StatefulSets(p.Namespace)
+	ssetClient := c.kclient.AppsV1().StatefulSets(p.Namespace)
 	// Ensure we have a StatefulSet running Prometheus deployed.
 	obj, exists, err = c.ssetInf.GetIndexer().GetByKey(prometheusKeyToStatefulSetKey(key))
 	if err != nil {
@@ -1099,11 +1178,11 @@ func ListOptions(name string) metav1.ListOptions {
 func PrometheusStatus(kclient kubernetes.Interface, p *monitoringv1.Prometheus) (*monitoringv1.PrometheusStatus, []v1.Pod, error) {
 	res := &monitoringv1.PrometheusStatus{Paused: p.Spec.Paused}
 
-	pods, err := kclient.Core().Pods(p.Namespace).List(ListOptions(p.Name))
+	pods, err := kclient.CoreV1().Pods(p.Namespace).List(ListOptions(p.Name))
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "retrieving pods of failed")
 	}
-	sset, err := kclient.AppsV1beta2().StatefulSets(p.Namespace).Get(statefulSetNameFromPrometheusName(p.Name), metav1.GetOptions{})
+	sset, err := kclient.AppsV1().StatefulSets(p.Namespace).Get(statefulSetNameFromPrometheusName(p.Name), metav1.GetOptions{})
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "retrieving stateful set failed")
 	}
@@ -1149,7 +1228,7 @@ func needsUpdate(pod *v1.Pod, tmpl v1.PodTemplateSpec) bool {
 	return false
 }
 
-func loadAdditionalScrapeConfigsSecret(additionalScrapeConfigs *v1.SecretKeySelector, s *v1.SecretList) ([]byte, error) {
+func (c *Operator) loadAdditionalScrapeConfigsSecret(additionalScrapeConfigs *v1.SecretKeySelector, s *v1.SecretList) ([]byte, error) {
 	if additionalScrapeConfigs != nil {
 		for _, secret := range s.Items {
 			if secret.Name == additionalScrapeConfigs.Name {
@@ -1160,7 +1239,10 @@ func loadAdditionalScrapeConfigsSecret(additionalScrapeConfigs *v1.SecretKeySele
 				return nil, fmt.Errorf("key %v could not be found in Secret %v", additionalScrapeConfigs.Key, additionalScrapeConfigs.Name)
 			}
 		}
-		return nil, fmt.Errorf("secret %v could not be found", additionalScrapeConfigs.Name)
+		if additionalScrapeConfigs.Optional == nil || !*additionalScrapeConfigs.Optional {
+			return nil, fmt.Errorf("secret %v could not be found", additionalScrapeConfigs.Name)
+		}
+		level.Debug(c.logger).Log("msg", fmt.Sprintf("secret %v could not be found", additionalScrapeConfigs.Name))
 	}
 	return nil, nil
 }
@@ -1235,6 +1317,15 @@ func loadBasicAuthSecret(basicAuth *monitoringv1.BasicAuth, s *v1.SecretList) (B
 
 }
 
+func gzipConfig(buf *bytes.Buffer, conf []byte) error {
+	w := gzip.NewWriter(buf)
+	defer w.Close()
+	if _, err := w.Write(conf); err != nil {
+		return err
+	}
+	return nil
+}
+
 func (c *Operator) loadBasicAuthSecrets(
 	mons map[string]*monitoringv1.ServiceMonitor,
 	remoteReads []monitoringv1.RemoteReadSpec,
@@ -1296,6 +1387,11 @@ func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus,
 		return errors.Wrap(err, "selecting ServiceMonitors failed")
 	}
 
+	pmons, err := c.selectPodMonitors(p)
+	if err != nil {
+		return errors.Wrap(err, "selecting PodMonitors failed")
+	}
+
 	sClient := c.kclient.CoreV1().Secrets(p.Namespace)
 	SecretsInPromNS, err := sClient.List(metav1.ListOptions{})
 	if err != nil {
@@ -1307,15 +1403,15 @@ func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus,
 		return err
 	}
 
-	additionalScrapeConfigs, err := loadAdditionalScrapeConfigsSecret(p.Spec.AdditionalScrapeConfigs, SecretsInPromNS)
+	additionalScrapeConfigs, err := c.loadAdditionalScrapeConfigsSecret(p.Spec.AdditionalScrapeConfigs, SecretsInPromNS)
 	if err != nil {
 		return errors.Wrap(err, "loading additional scrape configs from Secret failed")
 	}
-	additionalAlertRelabelConfigs, err := loadAdditionalScrapeConfigsSecret(p.Spec.AdditionalAlertRelabelConfigs, SecretsInPromNS)
+	additionalAlertRelabelConfigs, err := c.loadAdditionalScrapeConfigsSecret(p.Spec.AdditionalAlertRelabelConfigs, SecretsInPromNS)
 	if err != nil {
 		return errors.Wrap(err, "loading additional alert relabel configs from Secret failed")
 	}
-	additionalAlertManagerConfigs, err := loadAdditionalScrapeConfigsSecret(p.Spec.AdditionalAlertManagerConfigs, SecretsInPromNS)
+	additionalAlertManagerConfigs, err := c.loadAdditionalScrapeConfigsSecret(p.Spec.AdditionalAlertManagerConfigs, SecretsInPromNS)
 	if err != nil {
 		return errors.Wrap(err, "loading additional alert manager configs from Secret failed")
 	}
@@ -1324,6 +1420,7 @@ func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus,
 	conf, err := c.configGenerator.generateConfig(
 		p,
 		smons,
+		pmons,
 		basicAuthSecrets,
 		additionalScrapeConfigs,
 		additionalAlertRelabelConfigs,
@@ -1338,7 +1435,13 @@ func (c *Operator) createOrUpdateConfigurationSecret(p *monitoringv1.Prometheus,
 	s.ObjectMeta.Annotations = map[string]string{
 		"generated": "true",
 	}
-	s.Data[configFilename] = []byte(conf)
+
+	// Compress config to avoid 1mb secret limit for a while
+	var buf bytes.Buffer
+	if err = gzipConfig(&buf, conf); err != nil {
+		return errors.Wrap(err, "couldnt gzip config")
+	}
+	s.Data[configFilename] = buf.Bytes()
 
 	curSecret, err := sClient.Get(s.Name, metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
@@ -1411,6 +1514,49 @@ func (c *Operator) selectServiceMonitors(p *monitoringv1.Prometheus) (map[string
 	return res, nil
 }
 
+func (c *Operator) selectPodMonitors(p *monitoringv1.Prometheus) (map[string]*monitoringv1.PodMonitor, error) {
+	namespaces := []string{}
+	// Selectors might overlap. Deduplicate them along the keyFunc.
+	res := make(map[string]*monitoringv1.PodMonitor)
+
+	podMonSelector, err := metav1.LabelSelectorAsSelector(p.Spec.PodMonitorSelector)
+	if err != nil {
+		return nil, err
+	}
+
+	// If 'PodMonitorNamespaceSelector' is nil only check own namespace.
+	if p.Spec.PodMonitorNamespaceSelector == nil {
+		namespaces = append(namespaces, p.Namespace)
+	} else {
+		podMonNSSelector, err := metav1.LabelSelectorAsSelector(p.Spec.PodMonitorNamespaceSelector)
+		if err != nil {
+			return nil, err
+		}
+
+		namespaces, err = c.listMatchingNamespaces(podMonNSSelector)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	level.Debug(c.logger).Log("msg", "filtering namespaces to select PodMonitors from", "namespaces", strings.Join(namespaces, ","), "namespace", p.Namespace, "prometheus", p.Name)
+
+	podMonitors := []string{}
+	for _, ns := range namespaces {
+		cache.ListAllByNamespace(c.pmonInf.GetIndexer(), ns, podMonSelector, func(obj interface{}) {
+			k, ok := c.keyFunc(obj)
+			if ok {
+				res[k] = obj.(*monitoringv1.PodMonitor)
+				podMonitors = append(podMonitors, k)
+			}
+		})
+	}
+
+	level.Debug(c.logger).Log("msg", "selected PodMonitors", "podmonitors", strings.Join(podMonitors, ","), "namespace", p.Namespace, "prometheus", p.Name)
+
+	return res, nil
+}
+
 // listMatchingNamespaces lists all the namespaces that match the provided
 // selector.
 func (c *Operator) listMatchingNamespaces(selector labels.Selector) ([]string, error) {
@@ -1428,6 +1574,7 @@ func (c *Operator) createCRDs() error {
 	crds := []*extensionsobj.CustomResourceDefinition{
 		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.Prometheus, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
 		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.ServiceMonitor, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
+		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.PodMonitor, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
 		k8sutil.NewCustomResourceDefinition(c.config.CrdKinds.PrometheusRule, c.config.CrdGroup, c.config.Labels.LabelsMap, c.config.EnableValidation),
 	}
 
@@ -1459,7 +1606,7 @@ func (c *Operator) createCRDs() error {
 	}{
 		{
 			monitoringv1.PrometheusesKind,
-			listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
 				return &cache.ListWatch{
 					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 						return c.mclient.MonitoringV1().Prometheuses(namespace).List(options)
@@ -1469,7 +1616,7 @@ func (c *Operator) createCRDs() error {
 		},
 		{
 			monitoringv1.ServiceMonitorsKind,
-			listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
 				return &cache.ListWatch{
 					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 						return c.mclient.MonitoringV1().ServiceMonitors(namespace).List(options)
@@ -1477,9 +1624,19 @@ func (c *Operator) createCRDs() error {
 				}
 			}).List,
 		},
+		{
+			monitoringv1.PodMonitorsKind,
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
+				return &cache.ListWatch{
+					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
+						return c.mclient.MonitoringV1().PodMonitors(namespace).List(options)
+					},
+				}
+			}).List,
+		},
 		{
 			monitoringv1.PrometheusRuleKind,
-			listwatch.MultiNamespaceListerWatcher(c.config.Namespaces, func(namespace string) cache.ListerWatcher {
+			listwatch.MultiNamespaceListerWatcher(c.logger, c.config.Namespaces, c.config.DeniedNamespaces, func(namespace string) cache.ListerWatcher {
 				return &cache.ListWatch{
 					ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 						return c.mclient.MonitoringV1().PrometheusRules(namespace).List(options)
